MfgAide = {}
MfgAide.isMisn = false --true if goods were loaded for the current mission; used to unload goods at end of mission
MfgAide.isWarned = false --true after warning has been shown at least once each misn
MfgAide.loadVol = nil --volume of goods loaded this trip
MfgAide.cargoVol = nil --how much cargo space ship used when launched
MfgAide.version = '1.0'

--## Choco ##--

MfgAide.ui, MfgAide.vars = dofile('ui.lua')

local function is_mfg_misn()
	local mission_name = GetActiveMissionInfo(1) or '' --TODO compatibility with more than one active mission
	return string.match(mission_name, "^Manufacture") or string.match(mission_name, "^Construct")
end

--Returns table of items to deliver for the active mission
local function get_items()
	local info = {GetActiveMissionInfo(1)}
	MfgAide.info = info --DEBUG
	
	if info[1] then
		local mission_name = info[1]
		
		local needed_items = {}
		for _,obj in ipairs(info[4] or {}) do
			local num_reqd, item_name, num_delivered = string.match((obj or {})[1] or "", "Deliver (%d+)x? (.+) to %P+ %u%-%d+%s*%[?(%d*)/?%d*%]?")
			local needed = (tonumber(num_reqd) or 0) - (tonumber(num_delivered) or 0) --remaining qty needed
			
			if needed>0 then
				if not needed_items[item_name] then
					needed_items[item_name]=needed
				else needed_items[item_name] = needed_items[item_name] + needed end --combine multiple entries
			end
		end
		
		return needed_items
	end
	
	return false --no mission data available
end

--Loads as many items as possible for the active mission to the active ship
--options.warn_abort - don't load cargo if not all items available
local function load_items(options)
	local options = options or {}
	local warn_abort = options.warn_abort or not MfgAide.vars.GetVars"IsLoadOnWarning"
	
	local needed_items = get_items()
	local available_items = {}
	local load_vol = 0 --volume of goods to be on active ship needed by misn
	
	if not needed_items then print'\127ffffffYou must have an active manufacturing mission to use MfgAide'; return end
	if not next(needed_items) and not MfgAide.isMisn then print'\127ffffffThis mission has no items to deliver'; return end
	
	--remove any items already loaded in ship from list
	if HasActiveShip() then
		local ship_cargo = GetShipInventory(GetActiveShipID()).cargo
		for _,itemid in ipairs(ship_cargo) do
			local name = GetInventoryItemName(itemid)
			local qty = GetInventoryItemQuantity(itemid) or 0
			local vol = GetInventoryItemVolume(itemid) or 0
			
			if needed_items[name] then
				load_vol = load_vol + math.min(needed_items[name], qty)*vol
				needed_items[name] = math.max(needed_items[name] - qty, 0)
				if needed_items[name]==0 then needed_items[name] = nil end
			end
		end
	else print'\127ffffffYou must have an active ship to use MfgAide'; return end
	
	local stationid = GetStationLocation()
	if stationid then
		local item_list = {}
		
		local sta_items = GetStationCargoList(stationid) --inventory at current station
		for _,itemid in ipairs(sta_items) do
			local name = GetInventoryItemName(itemid)
			local qty = GetInventoryItemQuantity(itemid) or 0
			local vol = GetInventoryItemVolume(itemid) or 0
			
			if needed_items[name] then
				local item = {itemid=itemid, name=name, qty=math.min(qty, needed_items[name]), vol=vol}
				table.insert(item_list, item)
				
				available_items[name] = (available_items[name] or 0) + qty
				
				needed_items[name] = math.max(needed_items[name] - qty, 0)
				if needed_items[name]==0 then needed_items[name] = nil end
			end
		end
		
		table.sort(item_list, function(a,b) return tonumber(a.vol)>tonumber(b.vol) end) --sort item_list, largest volume first
		MfgAide.list = item_list
		
		--identify any needed items that are not available
		local is_missing = false
		local missing_items = '\127ff0000MfgAide WARNING\127ffffff -- The following items are not available in inventory: '
		for name,num in pairs(needed_items) do
			missing_items = missing_items.." "..num.."x "..name..","
			is_missing = true
		end
		if is_missing then
			if not MfgAide.isWarned or not MfgAide.vars.GetVars"IsOneWarning" then
				print(missing_items)
				MfgAide.isWarned = true --only show warning once per mission --TODO add option
			elseif MfgAide.vars.GetVars"IsOneWarning" and not MfgAide.vars.GetVars"IsLoadOnWarning" then
				print'\127ffffffYour current setting prevent MfgAide from loading any items. To change them use: \127a0a0a0/mfg options\127o'
			end
			
			if warn_abort then return false end --don't load cargo if warning message
		end
		
		local max_cargo = GetActiveShipMaxCargo()
		local free_space = max_cargo - GetActiveShipCargoCount()
		
		local load_cargo = {}
		for _,item in ipairs(item_list) do
			local move_qty = math.min(math.floor(free_space/item.vol), item.qty)
			if move_qty>0 then
				table.insert(load_cargo, {itemid=item.itemid, quantity=move_qty})
				free_space = free_space - move_qty*item.vol
				load_vol = load_vol + move_qty*item.vol
				
				if free_space <=0 then break end --TODO any 0cu cargo exists???
			end
		end
		
		if load_vol>0 then
			MfgAide.loadVol = load_vol
			RegisterEvent(MfgAide, "LEAVING_STATION") --note how much cargo ship has when it departs
			
			LoadCargo(load_cargo) --TODO cancel dlg?
		end
		
		MfgAide.avail = available_items --DEBUG
		MfgAide.needed = needed_items --DEBUG
		
		return true
	else print'\127ffffffYou must be docked at a station to use MfgAide'; return end
end

function MfgAide:PLAYER_ENTERED_GAME(event, data)
	RegisterEvent(MfgAide, "SHOW_STATION")
	RegisterEvent(MfgAide, "HUD_SHOW")
end

function MfgAide:SHOW_STATION(event, data)
	UnregisterEvent(MfgAide, "SHOW_STATION")
	UnregisterEvent(MfgAide, "HUD_SHOW")
	
	local needed_items = get_items()
	local available_items = {}
	local load_vol = 0
	
	if not is_mfg_misn() or not next(needed_items) then --[[console_print'no_misn';]] return end
	
	if HasActiveShip() then
		local ship_cargo = GetShipInventory(GetActiveShipID()).cargo
		for _,itemid in ipairs(ship_cargo) do
			local name = GetInventoryItemName(itemid)
			local qty = GetInventoryItemQuantity(itemid) or 0
			local vol = GetInventoryItemVolume(itemid) or 0
			
			if needed_items[name] then
				load_vol = load_vol + math.min(needed_items[name], qty)*vol
				needed_items[name] = math.max(needed_items[name] - qty, 0)
				if needed_items[name]==0 then needed_items[name] = nil end
			end
		end
	else return end
	
	if load_vol>0 then--active ship has mission cargo loaded when entered game
		MfgAide.isMisn = true
		MfgAide.loadVol = load_vol
		
		if not GetStationLocation() then --player is not in station
			MfgAide.cargoVol = GetActiveShipCargoCount() --note amount of cargo loaded
			RegisterEvent(MfgAide, "INVENTORY_REMOVE")
		else RegisterEvent(MfgAide, "LEAVING_STATION") end --note amount of cargo when depart station
	elseif MfgAide.vars.GetVars"IsAutoLoad" then MfgAide.isMisn = load_items{warn_abort=true} end --auto-load more items if ship cargo empty
end

function MfgAide:HUD_SHOW(event, data)
	UnregisterEvent(MfgAide, "SHOW_STATION")
	UnregisterEvent(MfgAide, "HUD_SHOW")
	--check if mission active
	--check how much of cargo is for misn
	
	local needed_items = get_items()
	local available_items = {}
	local load_vol = 0
	
	if not is_mfg_misn() or not next(needed_items) then --[[console_print'no_misn';]] return end
	
	if HasActiveShip() then
		local ship_cargo = GetShipInventory(GetActiveShipID()).cargo
		for _,itemid in ipairs(ship_cargo) do
			local name = GetInventoryItemName(itemid)
			local qty = GetInventoryItemQuantity(itemid) or 0
			local vol = GetInventoryItemVolume(itemid) or 0
			
			if needed_items[name] then
				load_vol = load_vol + math.min(needed_items[name], qty)*vol
				needed_items[name] = math.max(needed_items[name] - qty, 0)
				if needed_items[name]==0 then needed_items[name] = nil end
			end
		end
	else return end
	
	if load_vol>0 then
		MfgAide.cargoVol = GetActiveShipCargoCount() --note amount of cargo loaded
		RegisterEvent(MfgAide, "INVENTORY_REMOVE")
	end
end

--Triggers after each item delivered. Once all items delivered load next set
function MfgAide:INVENTORY_REMOVE(event, data)
	if MfgAide.loadVol and MfgAide.cargoVol and GetStationLocation() then
		local cargo_min = math.max(MfgAide.cargoVol - MfgAide.loadVol, 0)
		local current_cargo = GetActiveShipCargoCount()
		
		if current_cargo <= cargo_min or current_cargo==0 then
			if MfgAide.vars.GetVars"IsAutoLoad" then MfgAide.isMisn = load_items{warn_abort=true} or MfgAide.isMisn end --do auto-load
			UnregisterEvent(MfgAide, "INVENTORY_REMOVE")
		end
	end
end

function MfgAide:LEAVING_STATION(event, data)
	MfgAide.cargoVol = GetActiveShipCargoCount()
	
	UnregisterEvent(MfgAide, "LEAVING_STATION")
	RegisterEvent(MfgAide, "INVENTORY_REMOVE")
end

function MfgAide:MISSION_ADDED(event, data)
	local info = {GetActiveMissionInfo(1)}
	MfgAide.ActiveInfo = info --DEBUG
	
	RegisterEvent(MfgAide, "MISSION_UPDATED") --wait until mission updates to continue
end

--Check that all required items are available when mission begins
function MfgAide:MISSION_UPDATED(event, data)
	UnregisterEvent(MfgAide, "MISSION_UPDATED")
	local needed_items = get_items()
	local available_items = {}
	
	if not is_mfg_misn() or not next(needed_items or {}) then --[[console_print'no objectives found';]] return end --no items to deliver on this mission
	
	if MfgAide.vars.GetVars"IsAutoLoad" then MfgAide.isMisn = load_items{warn_abort=true} or MfgAide.isMisn end --do auto-load
end

--Unload all cargo when misn completes
function MfgAide:MISSION_REMOVED(event, data)
	if MfgAide.isMisn then --only do unload if items were loaded this mission
		--unload all cargo --TODO find just the mfg item and unload it?
		if HasActiveShip() and MfgAide.vars.GetVars"IsAutoUnload" then --do auto-unload
			local unload_items = {}
			
			local ship_cargo = GetShipInventory(GetActiveShipID()).cargo
			for _,itemid in ipairs(ship_cargo) do
				local qty = GetInventoryItemQuantity(itemid) or 0
				table.insert(unload_items, {itemid=itemid, quantity=qty})
			end
			CheckStorageAndUnloadCargo(unload_items)
		end
		
		MfgAide.isMisn = false
		MfgAide.isWarned = false
		MfgAide.loadVol = nil
		MfgAide.cargoVol = nil
		UnregisterEvent(MfgAide, "INVENTORY_REMOVE")
	end
end

function MfgAide:PLAYER_LOGGED_OUT(event, data)
	MfgAide.isMisn = false
	MfgAide.isWarned = false
	MfgAide.loadVol = nil
	MfgAide.cargoVol = nil
	
	UnregisterEvent(MfgAide, "INVENTORY_REMOVE")
	UnregisterEvent(MfgAide, "MISSION_UPDATED")
	UnregisterEvent(MfgAide, "LEAVING_STATION")
end

MfgAide.Inputs = {load=true,options=true,help=true} --list of valid commands
function MfgAide:input(args)
	args = args or {}
	local command = args[1] and args[1]:lower()
	if command then table.remove(args, 1) end
	
	if MfgAide.Inputs[command] then
		MfgAide[command](MfgAide, args)
	else MfgAide.load(MfgAide) end
end

function MfgAide:load(args) MfgAide.isMisn = load_items() or MfgAide.isMisn end
function MfgAide:options(args) MfgAide.ui:Show() end
function MfgAide:help(args) MfgAide.ui:ShowHelp() end
--function MfgAide:help(args) print('\127ffffffMfgAide v'..MfgAide.version..': /mfg - load goods for current mission. Auto-loading enabled') end

RegisterEvent(MfgAide, "PLAYER_ENTERED_GAME")
RegisterEvent(MfgAide, "MISSION_ADDED")
RegisterEvent(MfgAide, "MISSION_REMOVED")
RegisterEvent(MfgAide, "PLAYER_LOGGED_OUT")
RegisterUserCommand("mfg", MfgAide.input)